@@ -1,15 +1,22 @@
 """
 Common utility classes and functions used throughout libtaxii.
-
 """
 
 from operator import attrgetter
-from StringIO import StringIO
-from re import sub as resub
-import dateutil.parser
-import random
+import re
+import sys
+from uuid import uuid4
 
+import dateutil.parser
 from lxml import etree
+import six
+
+try:
+    import simplejson as json
+except ImportError:
+    import json
+
+from libtaxii.constants import *
 
 _XML_PARSER = None
 
@@ -30,31 +37,54 @@ def parse(s):
     return e
 
 
+def parse_xml_string(xmlstr):
+    """Parse an XML string (binary or unicode) with the default parser.
+
+    :param xmlstr: An XML String to parse
+    :return: an etree._Element
+    """
+    if isinstance(xmlstr, six.binary_type):
+        xmlstr = six.BytesIO(xmlstr)
+    elif isinstance(xmlstr, six.text_type):
+        # LXML doesn't accept Unicode strings with an explicit encoding, so
+        # try to detect and encode to bytes before passing to LXML.
+        encoding = re.findall(r'encoding="([0-9A-Za-z_\-]+)"', xmlstr[:50], re.I)
+        # re.findall returns a list of matching strings. We only care about the
+        # first one.
+        if encoding:
+            xmlstr = six.BytesIO(xmlstr.encode(encoding[0]))
+        else:
+            xmlstr = six.StringIO(xmlstr)
+
+    return parse(xmlstr)
+
+
 def get_xml_parser():
     """Return the XML parser currently in use.
 
     If one has not already been set (via :py:func:`set_xml_parser()`), a new
     ``etree.XMLParser`` is constructed with ``no_network=True`` and
-    ``huge_tree=True``.
+    ``huge_tree=False``.
     """
     global _XML_PARSER
     if _XML_PARSER is None:
-        _XML_PARSER = etree.XMLParser(attribute_defaults=False,
-                                      dtd_validation=False,
-                                      load_dtd=False,
-                                      no_network=True,
-                                      ns_clean=True,
-                                      recover=False,
-                                      remove_blank_text=False,
-                                      remove_comments=False,
-                                      remove_pis=False,
-                                      strip_cdata=True,
-                                      compact=True,
-                                      # collect_ids=True,
-                                      resolve_entities=False,
-                                      huge_tree=False)
-
-    return _XML_PARSER
+        _XML_PARSER = etree.XMLParser(
+            attribute_defaults=False,
+            dtd_validation=False,
+            load_dtd=False,
+            no_network=True,
+            ns_clean=True,
+            recover=False,
+            remove_blank_text=False,
+            remove_comments=False,
+            remove_pis=False,
+            strip_cdata=True,
+            compact=True,
+            # collect_ids=True,
+            resolve_entities=False,
+            huge_tree=False
+        )
+    return _XML_PARSER.copy()
 
 
 def set_xml_parser(xml_parser=None):
@@ -77,7 +107,7 @@ def parse_datetime_string(datetime_string):
     return dateutil.parser.parse(datetime_string)
 
 
-def generate_message_id(maxlen=5):
+def generate_message_id(maxlen=5, version=VID_TAXII_SERVICES_10):
     """Generate a TAXII Message ID.
 
     Args:
@@ -91,8 +121,13 @@ def generate_message_id(maxlen=5):
             # Or...
             message = tm11.DiscoveryRequest(tm11.generate_message_id())
     """
-    message_id = random.randint(1, 10 ** maxlen)
-    return str(message_id)
+    if version == VID_TAXII_SERVICES_10:
+        message_id = str(uuid4().int % sys.maxsize)
+    elif version == VID_TAXII_SERVICES_11:
+        message_id = str(uuid4())
+    else:
+        raise ValueError('Unknown TAXII Version: %s. Must be a TAXII Services Version ID!' % version)
+    return message_id
 
 
 def append_any_content_etree(etree_elt, content):
@@ -117,8 +152,8 @@ def append_any_content_etree(etree_elt, content):
         etree_elt.append(content)
         return etree_elt
 
-    if not isinstance(content, basestring):  # If content is a non-string, cast it to string and set etree_elt.text
-        etree_elt.test = str(content)
+    if not isinstance(content, six.string_types):  # If content is a non-string, cast it to string and set etree_elt.text
+        etree_elt.text = str(content)
         return etree_elt
 
     # If content is a string, need to check if it's XML or not
@@ -140,12 +175,16 @@ def gen_filename(collection_name, format_part, date_string, extension):
     :param extension: The file extension to use
     :return: A string containing the generated filename
     """
-
-    filename = (collection_name.lstrip(".") +
+    if six.PY3:
+        return (collection_name.lstrip(".") +
                 format_part +
-                resub(r"[^a-zA-Z0-9]", "_", date_string) + extension
+                re.sub(r"[^a-zA-Z0-9]", "_", date_string) + extension
+                ).translate('/\\:*?"<>|')
+    else:
+        return (collection_name.lstrip(".") +
+                format_part +
+                re.sub(r"[^a-zA-Z0-9]", "_", date_string) + extension
                 ).translate(None, '/\\:*?"<>|')
-    return filename
 
 
 class TAXIIBase(object):
@@ -181,12 +220,23 @@ class TAXIIBase(object):
         """
         raise NotImplementedError()
 
+    def to_json(self):
+        """Create a JSON object of this class.
+
+        Assumes any binary content will be UTF-8 encoded.
+        """
+        content_dict = self.to_dict()
+
+        _decode_binary_fields(content_dict)
+
+        return json.dumps(content_dict)
+
     def to_xml(self, pretty_print=False):
         """Create an XML representation of this class.
 
         Subclasses should not need to implement this method.
         """
-        return etree.tostring(self.to_etree(), pretty_print=pretty_print)
+        return etree.tostring(self.to_etree(), pretty_print=pretty_print, encoding='utf-8')
 
     def to_text(self, line_prepend=''):
         """Create a nice looking (this is a subjective term!)
@@ -221,12 +271,7 @@ class TAXIIBase(object):
 
         Subclasses should not need to implement this method.
         """
-        if isinstance(xml, basestring):
-            xmlstr = StringIO(xml)
-        else:
-            xmlstr = xml
-
-        etree_xml = parse(xmlstr)
+        etree_xml = parse_xml_string(xml)
         return cls.from_etree(etree_xml)
 
     # Just noting that there is not a from_text() method. I also
@@ -254,25 +299,21 @@ class TAXIIBase(object):
         """
         if other is None:
             if debug:
-                print 'other was None!'
+                print('other was None!')
             return False
 
         if self.__class__.__name__ != other.__class__.__name__:
             if debug:
-                print 'class names not equal: %s != %s' % (self.__class__.__name__, other.__class__.__name__)
+                print('class names not equal: %s != %s' % (self.__class__.__name__, other.__class__.__name__))
             return False
 
         # Get all member properties that start with '_'
-        members = [attr for attr in dir(self) if not callable(attr) and attr.startswith('_') and not attr.startswith('__')]
+        members = [attr for attr in vars(self) if attr.startswith('_') and not attr.startswith('__')]
         for member in members:
-            # TODO: The attr for attr... statement includes functions for some strange reason...
-            if member not in self.__dict__:
-                continue
-
             if debug:
-                print 'member name: %s' % member
-            self_value = self.__dict__[member]
-            other_value = other.__dict__[member]
+                print('member name: %s' % member)
+            self_value = getattr(self, member)
+            other_value = getattr(other, member)
 
             if isinstance(self_value, TAXIIBase):
                 # A debuggable equals comparison can be made
@@ -296,7 +337,7 @@ class TAXIIBase(object):
                         # All TAXIIBase objects have the 'sort_key' property implemented
                         self_value = sorted(self_value, key=attrgetter('sort_key'))
                         other_value = sorted(other_value, key=attrgetter('sort_key'))
-                        for self_item, other_item in zip(self_value, other_value):
+                        for self_item, other_item in six.moves.zip(self_value, other_value):
                             # Compare the ordered lists element by element
                             eq = self_item.__eq__(other_item, debug)
                     else:
@@ -306,17 +347,17 @@ class TAXIIBase(object):
                 # Dictionary to compare
                 if len(set(self_value.keys()) - set(other_value.keys())) != 0:
                     if debug:
-                        print 'dict keys not equal: %s != %s' % (self_value, other_value)
+                        print('dict keys not equal: %s != %s' % (self_value, other_value))
                     eq = False
-                for k, v in self_value.iteritems():
+                for k, v in six.iteritems(self_value):
                     if other_value[k] != v:
                         if debug:
-                            print 'dict values not equal: %s != %s' % (v, other_value[k])
+                            print('dict values not equal: %s != %s' % (v, other_value[k]))
                         eq = False
                 eq = True
             elif isinstance(self_value, etree._Element):
                 # Non-TAXII etree element (i.e. STIX)
-                eq = (etree.tostring(self_value) == etree.tostring(other_value))
+                eq = (etree.tostring(self_value, encoding='utf-8') == etree.tostring(other_value, encoding='utf-8'))
             else:
                 # Do a direct comparison
                 eq = (self_value == other_value)
@@ -324,10 +365,73 @@ class TAXIIBase(object):
             # TODO: is this duplicate?
             if not eq:
                 if debug:
-                    print '%s was not equal: %s != %s' % (member, self_value, other_value)
+                    print('%s was not equal: %s != %s' % (member, self_value, other_value))
                 return False
 
         return True
 
     def __ne__(self, other, debug=False):
         return not self.__eq__(other, debug)
+
+
+def get_required(etree_xml, xpath, ns_map):
+    elements = etree_xml.xpath(xpath, namespaces=ns_map)
+    if len(elements) == 0:
+        raise ValueError('Element "%s" is required' % xpath)
+    return elements[0]
+
+
+def get_optional(etree_xml, xpath, ns_map):
+    try:
+        return get_required(etree_xml, xpath, ns_map)
+    except ValueError:
+        pass
+
+
+def get_optional_text(etree_xml, xpath, ns_map):
+    try:
+        return get_required(etree_xml, xpath, ns_map).text
+    except ValueError:
+        pass
+
+
+def _decode_binary_fields(dict_obj):
+    """Given a dict, decode any binary values, assuming UTF-8 encoding.
+    Will recurse into nested dicts.
+    Modifies the values in-place.
+    """
+    for key, value in dict_obj.items():
+
+        if isinstance(value, six.binary_type):
+            dict_obj[key] = value.decode('utf-8')
+
+        elif isinstance(value, dict):
+            _decode_binary_fields(value)
+
+
+def stringify_content(content):
+    """Always a string or raises an error.
+    Returns the string representation and whether the data is XML.
+    """
+    # If it's an etree, it's definitely XML
+    if isinstance(content, etree._ElementTree):
+        return content.getroot(), True
+
+    if isinstance(content, etree._Element):
+        return content, True
+
+    if hasattr(content, 'read'):  # The content is file-like
+        try:  # Try to parse as XML
+            xml = parse(content)
+            return xml, True
+        except etree.XMLSyntaxError:  # Content is not well-formed XML; just treat as a string
+            return content.read(), False
+    else:  # The Content is not file-like
+        try:  # Attempt to parse string as XML
+            xml = parse_xml_string(content)
+            return xml, True
+        except etree.XMLSyntaxError:  # Content is not well-formed XML; just treat as a string
+            if isinstance(content, six.string_types):  # It's a string of some kind, unicode or otherwise
+                return content, False
+            else:  # It's some other datatype that needs casting to string
+                return str(content), False
